#include <reg52.h>

sbit EN1 = P1^0; // L293D使能引脚1
sbit EN2 = P1^1; // L293D使能引脚2
sbit IN1 = P1^2; // L293D输入引脚1
sbit IN2 = P1^3; // L293D输入引脚2
sbit IN3 = P1^4; // L293D输入引脚3
sbit IN4 = P1^5; // L293D输入引脚4

sbit LED = P2^3; // LED灯
sbit BUZZER = P2^7; // 喇叭

void delay(unsigned int t) // 延时函数
{
    unsigned int i, j;
    for(i=0; i<t; i++)
        for(j=0; j<125; j++);
}

void forward() // 小车前进
{
    EN1 = 1;
    EN2 = 1;
    IN1 = 1;
    IN2 = 0;
    IN3 = 1;
    IN4 = 0;
}

void backward() // 小车后退
{
    EN1 = 1;
    EN2 = 1;
    IN1 = 0;
    IN2 = 1;
    IN3 = 0;
    IN4 = 1;
}

void left() // 小车左转
{
    EN1 = 1;
    EN2 = 1;
    IN1 = 0;
    IN2 = 1;
    IN3 = 1;
    IN4 = 0;
}

void right() // 小车右转
{
    EN1 = 1;
    EN2 = 1;
    IN1 = 1;
    IN2 = 0;
    IN3 = 0;
    IN4 = 1;
}

void stop() // 小车停止
{
    EN1 = 0;
    EN2 = 0;
    IN1 = 0;
    IN2 = 0;
    IN3 = 0;
    IN4 = 0;
}

void main()
{
    unsigned char data;
    TMOD = 0x20; // 定时器1工作在方式2
    TH1 = 0xfd; // 波特率9600
    TL1 = 0xfd;
    SCON = 0x50; // 串口工作在方式1
    TR1 = 1; // 启动定时器1
    ES = 1; // 使能串口中断
    EA = 1; // 使能总中断
    LED = 0; // 初始化LED灯
    BUZZER = 0; // 初始化喇叭
    while(1)
    {
        if(data == 'F') // 前进
            forward();
        else if(data == 'B') // 后退
            backward();
        else if(data == 'L') // 左转
            left();
        else if(data == 'R') // 右转
            right();
        else if(data == 'S') // 停止
            stop();
        else if(data == 'O') // 打开LED灯
            LED = 1;
        else if(data == 'C') // 关闭LED灯
            LED = 0;
        else if(data == 'P') // 播放喇叭
            BUZZER = 1;
        else if(data == 'Q') // 停止喇叭
            BUZZER = 0;
        delay(10); // 延时一段时间
    }
}

void serial() interrupt 4 // 串口中断服务程序
{
    if(RI) // 接收到数据
    {
        RI = 0;
        data = SBUF; // 保存数据
    }
}